// $Id: $
#ifndef PREPAREVELOFULLRAWBUFFER_H 
#define PREPAREVELOFULLRAWBUFFER_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"

// STL
#include <vector>

// data model
#include "Kernel/VeloChannelID.h"
#include "DecodeCore.h"
#include "VeloFullBank.h"

/** @class PrepareVeloFullRawBuffer PrepareVeloFullRawBuffer.h
 *  
 *
 *  @author Tomasz Szumlak
 *  @date   2006-03-22
 *
 *  Algorithm dedicated to decoding the new 1MHz non-zero suppresed
 *  RawBank which is sent out by TELL1 pre-processing board.
 *  The input for the algorithm is binary file and the output 
 *  is container with VeloDigits objects.
 *  The container is the input for the TELL1
 *  algorithms (pedestal subtraction and following, FIR filter nad
 *  linear CM subtraction)
 */

namespace LHCb
{
  class RawEvent;
  class RawBank;
}
class DeVelo;
class DeVeloSensor;

using namespace DecodeCore;

class PrepareVeloFullRawBuffer : public GaudiTupleAlg {
public: 

  // define structure of the data sections
  enum sectionShifts{
    ADCHeadersShift=4,
    ALinkShift=36
  };
  enum bitMasks{
    ADCMask=0x000003ff
  };  
  enum bitShifts{
    ADCShift=10
  };  
  //
  /// Standard constructor
  PrepareVeloFullRawBuffer( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~PrepareVeloFullRawBuffer( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization
  // fetch the RawEvent
  StatusCode getRawEvent();
  // extract vector of the RawBanks of the type VeloFull (non-zero supp.)
  StatusCode getRawBanks();
  // get the number of the read-out sensors == number of TELL1s
  void setNumberOfSensors(int sensors);
  int numberOfSensors();
  // separate the data stream 
  void createOrderedSections(int nbOfBank);
  // write-out the Gaudi model of the RawBank (with digit objects,
  // header infos and Beetle headers)
  StatusCode writeVeloFull();
  // return number of TELL1 source (sensor) ID
  int sourceID(int nbOfBank);
  // run after decoding each event
  void resetMemory();
  // use only as emergency checker!
  void dumpADCs(const dataVec& inADCs);
  // return pointer to the data bank 
  unsigned int* data(int noOfSensor);
  // return poiter to the pedestal bank
  unsigned int* pedestals(int noOfSensor);

protected:

  void setADCBankFlag();
  void setPedBankFlag();
  bool adcBankFlag();
  bool pedBankFlag();

private:

  bool m_setMainLoop;
  LHCb::RawEvent* m_rawEvent;          /// pointer to RawEvent container
  rawVec m_data;                       /// pointer to non-zero suppressed data
  rawVec m_ped;                        /// pointer to pedestals
  int m_numberOfSensors;
  std::vector<int> m_sensors;          /// vector to keep sensors number
  //  std::vector<VeloFullBank> m_bankPed; 
  int m_firstChannel;                  /// histogram handling
  int m_lastChannel;
  int m_ADCLowLimit;
  int m_ADCHighLimit;
  int m_channelBins;
  int m_ADCBins;
  DeVelo* m_veloDet;                   /// pointer to Velo detector element
  const DeVeloSensor* m_veloSens;      /// pointer to Velo sensor
  VeloFullBanks* m_veloADCData;        /// container to store ordered data
  VeloFullBanks* m_veloPedestals;      /// container to store ordered pedestals
  std::string m_veloADCDataContainer;
  std::string m_veloPedestalsContainer;
  bool m_adcBankPresent;               /// flag to check if data bank is sent out
  bool m_pedBankPresent;               /// flag to check if ped bank is sent out
  
};
#endif // DECODEVELOFULLRAWBUFFER_H
