// $Id: $
// Include files 

// from Gaudi
#include "GaudiKernel/DeclareFactoryEntries.h" 

// local
#include "TELL1BinaryOutputChecker.h"

// event model
#include "Event/RawBank.h"

//-----------------------------------------------------------------------------
// Implementation file for class : TELL1BinaryOutputChecker
//
// 2006-03-22 : Tomasz Szumlak
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( TELL1BinaryOutputChecker );

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
TELL1BinaryOutputChecker::TELL1BinaryOutputChecker( const std::string& name,
                                                    ISvcLocator* pSvcLocator)
  : GaudiAlgorithm ( name , pSvcLocator )
{ }
//=============================================================================
// Destructor
//=============================================================================
TELL1BinaryOutputChecker::~TELL1BinaryOutputChecker() {}
//=============================================================================
// Initialization
//=============================================================================
StatusCode TELL1BinaryOutputChecker::initialize() {
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by GaudiAlgorithm

  debug() << "==> Initialize" << endmsg;
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
// Main execution
//=============================================================================
StatusCode TELL1BinaryOutputChecker::execute() {

  debug() << "==> Execute" << endmsg;
  //
  info()<< "Getting Raw Bank!" <<endmsg;  
  StatusCode sc=getRawBank();
  //
  return sc;
}
//=============================================================================
//  Finalize
//=============================================================================
StatusCode TELL1BinaryOutputChecker::finalize() {

  debug() << "==> Finalize" << endmsg;

  return GaudiAlgorithm::finalize();  // must be called after all other actions
}
//=============================================================================
StatusCode TELL1BinaryOutputChecker::getRawBank()
{
    debug()<< "===> executing getRawBank routine" <<endmsg;
    debug()<< "---------------------------------" <<endmsg;
  //
  if(!exist<LHCb::RawEvent>(LHCb::RawEventLocation::Default)){
    info()<< " ==> There is no RawEvent at: "
          << LHCb::RawEventLocation::Default <<endmsg;
  }
  // get the RawEvent from default TES location
  m_rawEvent=get<LHCb::RawEvent>(LHCb::RawEventLocation::Default);
  info()<< " ==> The RawEvent has been read-in from location: "
        << LHCb::RawEventLocation::Default <<endmsg;
  //
  for(int i=0; i<LHCb::RawBank::LastType; ++i){
    LHCb::RawBank::BankType type=LHCb::RawBank::BankType(i);
    const std::vector<LHCb::RawBank*>& banks=
          m_rawEvent->banks(type);
    if(banks.size()!=0){
      info()<< " ==> bank size: " << banks.size() <<endmsg;
      std::vector<LHCb::RawBank*>::const_iterator bankIt;
      for(bankIt=banks.begin(); bankIt!=banks.end(); bankIt++){
        const LHCb::RawBank* aBank=(*bankIt);
        info()<< " size: " << int(aBank->size()) << ", "
              << " sourceID: " << int(aBank->sourceID()) << ", "
              << " version: " << int(aBank->version()) << ", "
              << " bank type: " << type << ", "
              << " magic pattern: "
              << std::hex << aBank->magic() << std::dec 
              << " pointer to the data body: "
              << aBank->data() <<endmsg;
        const unsigned int* ptrToData=aBank->data();
        const unsigned int* ptrDat=ptrToData+224;
        info()<< "dec data: " << (((*ptrDat))&0x000003ff) <<endmsg;
      }
    }
  }
  //
  return ( StatusCode::SUCCESS );
}
