// $Id: $
#ifndef VELOFULLBANK_H 
#define VELOFULLBANK_H 1

// Include files
#include "GaudiKernel/KeyedObject.h"
#include "GaudiKernel/KeyedContainer.h"
#include "Event/VeloDigit.h"
#include "DecodeConf.h"
#include "DecodeCore.h"

/** @class VeloFullBank VeloFullBank.h
 *  
 *
 *  @author Tomasz Szumlak
 *  @date   2006-04-06
 */

using namespace DecodeCore;

static const CLID& CLID_VeloFullBank = 4005;

namespace VeloFullBankLocation {
  static const std::string& Default = "Raw/Velo/ADCBank";
  static const std::string& Pedestals = "Raw/Velo/PedBank";
}

class VeloFullBank: public KeyedObject<int>{

public: 
  /// Standard constructor
  typedef std::vector<unsigned int> section;
  //
  VeloFullBank(const int numberOfTELL1, unsigned int* inBank, const int type):
    KeyedObject<int>(numberOfTELL1)
    {
      initialize(type);
      m_bank=inBank;
      writeSections();
      writeEventInfos(type);
    }
  virtual ~VeloFullBank( ) { }; ///< Destructor
  virtual const CLID& clID() const;
  static const CLID& classID();
  dataVec& getSection(int blockNnb, int secNbn);
  allEvt& getEvtInfo();
  //

protected:  

  void initialize(const int type);
  void writeSections();
  void writeEventInfos(const int type);
  const int NumberOfWordsInSection() const;
  const int NumberOfWordsInBlock() const;

private:

  struct PPFPGABlock
  {
    std::vector<section> sections;
  };
  unsigned int* m_bank;
  std::vector<PPFPGABlock> m_PPFPGABlocks;
  int m_NumberOfWordsInBlock;
  int m_NumberOfWordsInSection;
  allEvt m_infos;
  
};
//=============================================================================
inline const CLID& VeloFullBank::clID() const
{
  return VeloFullBank::classID();
}
//=============================================================================
inline const CLID& VeloFullBank::classID()
{
  return CLID_VeloFullBank;
}
//=============================================================================
inline void VeloFullBank::initialize(const int type)
{
  if(type==VeloFullConf::VeloFull){
    m_NumberOfWordsInBlock=(VeloFullConf::PPFPGADataBlock);
    m_NumberOfWordsInSection=(VeloFullConf::PPFPGADataSection); 
  }
  else if(type==VeloFullConf::VeloPedestal){
    m_NumberOfWordsInBlock=VeloFullConf::PPFPGAPedBlock;
    m_NumberOfWordsInSection=VeloFullConf::PPFPGAPedBlock; 
  }  
}
//=============================================================================
inline void VeloFullBank::writeSections()
{
  // set length of the block
  const int wordsInBlock=NumberOfWordsInBlock();
  // skip the last eight words which contain event info
  const int dataWords=NumberOfWordsInSection();
  //                                                
  for(int i=0; i<VeloFullConf::NumberOfPPFPGA; i++){
    // we can divide data stream form the bank on four logical
    // units each from which corrsponds to data that will be sent
    // to one PPFPGA processor
    PPFPGABlock aBlock;
    // set the pointer at the beginning of the PPFPGA block
    unsigned int* dataPtr=m_bank+i*wordsInBlock;
    for(int k=0; k<VeloFullConf::SectionsPerBlock; k++){
      // create section object, there are three sections 
      // per one data block
      section sec;
      unsigned int* secPtr=dataPtr+k;
      int ii=0;
      while(ii<dataWords){
        sec.push_back((*(secPtr+ii)));
        ii+=3; 
      }
      aBlock.sections.push_back(sec); 
    }
    m_PPFPGABlocks.push_back(aBlock); 
  } 
}
//============================================================================
inline void VeloFullBank::writeEventInfos(const int type)
{
  if(type==VeloFullConf::VeloFull){
    int DataBlock=VeloFullConf::PPFPGADataBlock;
    int DataSection=VeloFullConf::PPFPGADataSection;
    //
    for(int i=0; i<VeloFullConf::NumberOfPPFPGA; i++){
      unsigned int* dataPtr=m_bank+(i*DataBlock)+DataSection;
      evtInfo anInfo;
      for(int k=0; k<VeloFullConf::EventInfoLength; k++){
        unsigned int* infoPtr=dataPtr+k;
        anInfo.push_back(*(infoPtr));
      }
      m_infos.push_back(anInfo);
    }
  }
}
//=============================================================================
inline const int VeloFullBank::NumberOfWordsInBlock() const
{
  return ( m_NumberOfWordsInBlock ); 
}
//=============================================================================
inline const int VeloFullBank::NumberOfWordsInSection() const
{
  return ( m_NumberOfWordsInSection ); 
}
//=============================================================================
inline allEvt& VeloFullBank::getEvtInfo()
{
  return ( m_infos );
}
//=============================================================================
inline dataVec& VeloFullBank::getSection(
                                            int blockNbn, int secNbn)
{
  return ( m_PPFPGABlocks[blockNbn].sections[secNbn] );
}
typedef KeyedContainer<VeloFullBank, Containers::HashMap> VeloFullBanks;
#endif // VELOFULLBANK_H
