// $Id: $
#ifndef VELOTELL1DATA_H 
#define VELOTELL1DATA_H 1

// Include files
#include "GaudiKernel/KeyedObject.h"
#include "GaudiKernel/KeyedContainer.h"
#include "DecodeCore.h"

/** @class VeloTELL1Data VeloTELL1Data.h
 *  
 *
 *  @author Tomasz Szumlak
 *  @date   2006-04-24
 */

using namespace DecodeCore;

namespace VeloTELL1DataLocation 
{
  static const std::string& ADCs = "Raw/Velo/DecodedADC";
  static const std::string& Pedestals = "Raw/Velo/DecodedPed";
  static const std::string& Headers = "Raw/Velo/DecodedHeaders";
}

class VeloTELL1Data: public KeyedObject<int>{

  enum objects{
    headers=4,
    adcs=32
  };
  //
  enum dataToWriteOut{
    writeHeaders=40
  };

public: 
  /// Standard constructor
  VeloTELL1Data(const int numberOfTELL1, const int dataType):
    KeyedObject<int>(numberOfTELL1),
    m_decodedData (),
    m_ALink (),
    m_dataType ( dataType )
  { }

  virtual ~VeloTELL1Data( ){ }; ///< Destructor
  void setDecodedData(dataVec& inVec);
  dataVec& operator[](const int ALinkNumber);

protected:

private:

  dataVec m_decodedData;
  dataVec m_ALink;
  int m_dataType;

};
//==============================================================================
inline void VeloTELL1Data::setDecodedData(dataVec& inVec)
{
  m_decodedData=inVec;
}
//==============================================================================
inline dataVec& VeloTELL1Data::operator[](const int ALinkNumber)
{
  // calculate starting position, check if data object contain
  // headers or ADC or pedestals, for the latter two configuration
  // is the same (32 objects per given link)
  int ALinkBegin=0, ALinkEnd=0;
  if(m_dataType==writeHeaders){
    ALinkBegin=ALinkNumber*headers;
    ALinkEnd=ALinkBegin+headers;
  }else{
    ALinkBegin=ALinkNumber*adcs;
    ALinkEnd=ALinkBegin+adcs;
  }
  // 
  m_ALink.clear();
  dataVec::iterator datIt;
  for(datIt=(m_decodedData.begin()+ALinkBegin);
      datIt!=(m_decodedData.begin()+ALinkEnd);
      datIt++){
    m_ALink.push_back(*datIt);
  }
  //
  return ( m_ALink );
}
//
typedef KeyedContainer<VeloTELL1Data, Containers::HashMap> VeloTELL1Datas;
#endif // VELOTELL1DATA_H
